@@ -195,7 +195,7 @@ prepare_rectangle (HTMLPainter *painter,
 	y = SCALE_ENGINE_TO_GNOME_PRINT (_y);
 	cr = gtk_print_context_get_cairo_context (context);
 	cairo_new_path (cr);
-	cairo_rectangle (cr, x, y, x + width, y + height);
+	cairo_rectangle (cr, x, y, x + width, y + height );
 	cairo_close_path (cr);
 }
 
@@ -334,7 +334,7 @@ draw_border (HTMLPainter *painter,
 	cairo_move_to  (cr, x, y);
 	cairo_line_to  (cr, x + width, y);
 	cairo_line_to  (cr, x + width - bs, y + bs);
-	cairo_line_to  (cr, x + bs, y + bs);
+	cairo_line_to  (cr, x + bs, y + bs );
 	cairo_line_to  (cr, x + bs, y + height - bs);
 	cairo_line_to  (cr, x, y + height);
 	cairo_close_path (cr);
@@ -474,13 +474,11 @@ static void
 draw_lines (HTMLPrinter *printer,
             double x,
             double y,
-            const PangoRectangle log_rect,
+            double width,
             PangoAnalysis *analysis,
             HTMLPangoProperties *properties)
 {
 	PangoFontMetrics *metrics;
-	gint dsc, asc;
-	gdouble width;
 	cairo_t *cr;
 
 	if (!properties->underline && !properties->strikethrough)
@@ -490,26 +488,26 @@ draw_lines (HTMLPrinter *printer,
 	cr = gtk_print_context_get_cairo_context (printer->context);
 	cairo_set_line_cap (cr, CAIRO_LINE_CAP_BUTT);
 
-	width = pango_units_to_double (log_rect.width);
-	dsc = PANGO_PIXELS (PANGO_DESCENT (log_rect));
-	asc = PANGO_PIXELS (PANGO_ASCENT (log_rect));
-
 	if (properties->underline) {
 		gdouble thickness = pango_units_to_double (pango_font_metrics_get_underline_thickness (metrics));
+		gdouble position = pango_units_to_double (pango_font_metrics_get_underline_position (metrics));
+		gdouble ly = y + position - thickness / 2;
 
 		cairo_new_path (cr);
-		cairo_move_to (cr, x, y + dsc - 2);
-		cairo_line_to (cr, x + width, y + dsc - 2);
+		cairo_move_to (cr, x, ly + 4);
+		cairo_line_to (cr, x + width, ly + 4);
 		cairo_set_line_width (cr, thickness);
 		cairo_stroke (cr);
 	}
 
 	if (properties->strikethrough) {
 		gdouble thickness = pango_units_to_double (pango_font_metrics_get_strikethrough_thickness (metrics));
+		gdouble position = pango_units_to_double (pango_font_metrics_get_strikethrough_position (metrics));
+		gdouble ly = y + position - thickness / 2;
 
 		cairo_new_path (cr);
-		cairo_move_to (cr, x, y - asc + (asc + dsc) / 2);
-		cairo_line_to (cr, x + width, y - asc + (asc + dsc) / 2);
+		cairo_move_to (cr, x, ly - 8);
+		cairo_line_to (cr, x + width, ly - 8);
 		cairo_set_line_width (cr, thickness);
 		cairo_stroke (cr);
 	}
@@ -568,7 +566,7 @@ draw_glyphs (HTMLPainter *painter,
 				       item->analysis.font,
 				       glyphs);
 	draw_lines (printer, print_x, print_y,
-		    log_rect,
+		    pango_units_to_double (log_rect.width),
 		    &item->analysis, &properties);
 
 	cairo_restore (cr);
